type comparison = Less | Equal | Greater

(* Creates a set of integers stored as as sorted list. *)
module IntSet =
struct
  type element = int
  let int_compare (i:int)(j:int):comparison = 
	 if i=j then Equal
	 else if i<j then Less
	 else Greater

  type set = element list

  let empty = []

  let rec add (x:element) (s:set) :set =
    match s with
        [] -> [x]
      | hd::tl ->
          match int_compare x hd with
              Equal   -> s         (* x is already in s *)
            | Less    -> x :: s    (* x is smaller than all 
                                        elements of s *)
            | Greater -> hd :: add x tl

  let rec member (x:element) (s:set):bool =
    match s with
		  [] -> false
      | hd::tl ->
          match int_compare x hd with
              Equal   -> true     (* x belongs to s *)
            | Less    -> false    (* x is smaller than all 
                                       elements of s *)
            | Greater -> member x tl

  let is_empty (s:set) = 
	 match s with [] -> true | _ -> false
end

let test_run = 
  let set = IntSet.add 2 (IntSet.add 3 (IntSet.add 4 IntSet.empty))
  in IntSet.member 4 set || IntSet.is_empty set

;;



