module type ORDERED_TYPE =
   sig
     type t
     val compare: t -> t -> comparison
   end;;

module IntElt = 
  struct 
	 type t = int 
	 let compare (i:t)(j:t):comparison = 
		if i=j then Equal
		else if i<j then Less
		else Greater
  end 

module SetFunctor =
   functor (Elt: ORDERED_TYPE) ->
     struct
       type element = Elt.t
       type set = element list
       let empty = []

       let rec add (x:element) (s:set) :set =
         match s with
           [] -> [x]
         | hd::tl ->
            match Elt.compare x hd with
              Equal   -> s         (* x is already in s *)
            | Less    -> x :: s    (* x is smaller than 
				      all elements of s *)
            | Greater -> hd :: add x tl

       let rec member (x:element) (s:set) :bool =
         match s with
           [] -> false
         | hd::tl ->
             match Elt.compare x hd with
               Equal   -> true     (* x belongs to s *)
             | Less    -> false    (* x is smaller than 
                                      all elements of s *)
             | Greater -> member x tl

		 let is_empty (s:set) :bool = 
			match s with [] -> true | _ -> false

     end;;

module IntSet3 = SetFunctor(IntElt);;


